#!/bin/sh
#
# install_etcfiles.sh
#
# $Id: install_etcfiles.sh,v 1.6 2006/12/07 10:33:42 root Exp root $

OS=unix

ID=`id -un`
if [ "$ID" != root ]
then
   echo
   echo "***  You must be root but you appear to be $ID."
   echo
   exit 1
fi

CFGDIR="$1"
if [ ! -d "$CFGDIR" ]
then
   echo "install_etcfiles.sh: $CFGDIR is not a directory"
   exit 1
fi

BINDIR="$2"
if [ ! -d "$BINDIR" ]
then
   echo "install_etcfiles.sh: $BINDIR is not a directory"
   exit 1
fi

if [ -d /usr/pkg/etc/rc.d ]
then
   OS=netbsd
   INITDIR=/usr/pkg/etc/rc.d
else
   if [ -d /usr/local/etc/rc.d ]		# freebsd 
   then
      OS=freebsd
      INITDIR=/usr/local/etc/rc.d
   else
      if [ -d /etc/rc.d/init.d ]		# linux 2.6
      then 
	 INITDIR=/etc/rc.d/init.d
	 if [ -d /etc/rc.d/rc3.d ]
	 then
	    RCROOT=/etc/rc.d
	 else
	    RCROOT=$INITDIR
	 fi
      else
	 if [ -d /etc/init.d ]			# linux 2.4, Solaris 9
	 then 
	    INITDIR=/etc/init.d
	    if [ -d /etc/rc.d/rc3.d ]
	    then
	       RCROOT=/etc/rc.d			# Solaris 9
	    else
	       RCROOT=$INITDIR 			# linux 2.4
	    fi
	 else
	    if [ -d /sbin/init.d ]		# HPUX 11.x
	    then
	       INITDIR=/sbin/init.d
	       RCROOT=$INITDIR
	    else
	       echo "cannot find the system rc init.d directory"
	       echo "please run install_etcfiles.sh manually and specify the directory"
	       exit 1
	    fi
	 fi
      fi
   fi
fi


install_netbsd () {

   if [ ! -w $INITDIR ]
   then
      echo "No permission to write to $INITDIR"
      exit 1
   fi

   rm -f $INITDIR/ufdb.sh
   cp mtserver/ufdb.sh $INITDIR/ufdb.sh
   chmod 755 $INITDIR/ufdb.sh

   echo "ufdb.sh copied to $INITDIR"
}


install_freebsd () {

   if [ ! -w $INITDIR ]
   then
      echo "No permission to write to $INITDIR"
      exit 1
   fi

   rm -f $INITDIR/ufdb.sh
   cp mtserver/ufdb.sh $INITDIR/ufdb.sh
   chmod 755 $INITDIR/ufdb.sh
}


install_unix () {

   if [ ! -w $INITDIR ]
   then
      echo "No permission to write to $INITDIR"
      exit 1
   fi

   rm -f $INITDIR/ufdb
   cp mtserver/ufdb.sh $INITDIR/ufdb
   chmod 755 $INITDIR/ufdb

   if [ ! -f $RCROOT/rc3.d/K02ufdb ]		# rc3
   then
      ln -s $INITDIR/ufdb $RCROOT/rc3.d/K02ufdb
   fi
   if [ ! -f $RCROOT/rc3.d/S50ufdb ]
   then
      ln -s $INITDIR/ufdb $RCROOT/rc3.d/S50ufdb
   fi

   if [ ! -f $RCROOT/rc5.d/K02ufdb ]		# rc5
   then
      ln -s $INITDIR/ufdb $RCROOT/rc5.d/K02ufdb
   fi
   if [ ! -f $RCROOT/rc5.d/S50ufdb ]
   then
      ln -s $INITDIR/ufdb $RCROOT/rc5.d/S50ufdb
   fi
}


case $OS in
   netbsd)      install_netbsd ;;
   freebsd)     install_freebsd ;;
   *) 		install_unix ;;
esac


if ! grep -q ufdbguardd /etc/services
then
   (
      echo ""
      echo "# for URLfilterDB daemon : "
      echo "ufdbguardd      3977/tcp"
   ) >> /etc/services
fi

