/*
  By accepting this notice, you agree to be bound by the following
  agreements:
  
  ufdbGuard is copyrighted (C) 2005,2006 by URLfilterDB with all rights reserved.
  ufdbGuard is based on squidGuard.

  squidGuard is copyrighted (C) 1998 by
  ElTele st AS, Oslo, Norway, with all rights reserved.
  
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License (version 2) as
  published by the Free Software Foundation.  It is distributed in the
  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
  PURPOSE.  See the GNU General Public License (GPL) for more details.
  
  You should have received a copy of the GNU General Public License
  (GPL) along with this program.

  $Id: sgLog.c,v 1.19 2006/12/07 10:35:18 root Exp root $
*/

#include "sg.h"
#include "ufdblib.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <pthread.h>

extern int    globalFatalError; 
extern int    globalPid;      /* from main.c */
extern char * globalLogDir;   /* from main.c */
extern struct LogFileStat * globalErrorLog;

static pthread_mutex_t log_mutex = PTHREAD_MUTEX_INITIALIZER;
static int    globalLogging = 1;


void ufdbGlobalSetLogging( int logging )
{
   globalLogging = logging;
}


void sgSetGlobalErrorLogFile( void )
{
  char file[1024];

  if (!globalLogging)
     return;

  if (UFDBglobalDebug)
    return;

  if (globalLogDir == NULL)
    strcpy( file, DEFAULT_LOGDIR );
  else
    strcpy( file, globalLogDir );
  strcat( file, "/" );

  if (progname != NULL  &&  progname[0] != '\0')
  {
     strcat( file, progname );
     strcat( file, ".log" );
  }
  else
     strcat( file, DEFAULT_LOGFILE );

  globalErrorLog = ufdbLogFileStat( file );
}


#ifdef UFDB_ROTATE_LOG_FILES
/* Rotate log files.
 * There is a race condition when multiple instance try to rotate.
 * Therefore acquire a lock and sleep for 1 second if the lock fails.
 */
void sgRotateLogfile( char * filename )
{
   int         i;
   int         lock;
   struct stat fbuf;
   char        oldfile[1024];
   char        newfile[1024];

#if 0
   fprintf( stderr, "sgRotateLogfile %s\n", filename );
#endif

   lock = open( filename, O_RDWR|O_CREAT, S_IRUSR|S_IWUSR );
   if (lock < 0)
      return;

   if (lockf( lock, F_TLOCK, 1 ) < 0)
   {
      /* someone else is doing the rotation...
       * wait until the lock is released...
       */
      lockf( lock, F_LOCK, 1 );

      close( lock );
      return;
   }

   /* in case that we are very late and missed the race condition
    * the files are already rotated !
    *
    * doublecheck to see if the rotation was already done.
    */
   stat( filename, &fbuf );
   if (fbuf.st_size < UFDB_MAX_INITIAL_LOGFILE_SIZE)
   {
      close( lock );
      return;
   }

   /* rotate the log file:
    * file.log.4  ->  file.log.5
    * file.log.3  ->  file.log.4
    * file.log.2  ->  file.log.3
    * file.log.1  ->  file.log.2
    * file.log    ->  file.log.1
    */

   for (i = 8; i > 1; i--)
   {
      sprintf( newfile, "%s.%d", filename, i );
      sprintf( oldfile, "%s.%d", filename, i-1 );
      rename( oldfile, newfile );
   }

   sprintf( newfile, "%s.%d", filename, 1 );
   rename( filename, newfile );

   close( lock );
}
#endif


void ufdbLog( struct LogFileStat * log, char * format, ... )
{
  FILE *  fd;
  va_list ap;
  long    pos;
  char    date[22];
#ifdef UFDB_ROTATE_LOG_FILES
  static  int sizecheckcounter = 0;	/* this var is OK in multithreaded app */
#endif
  char    msg[MAX_BUF];

  if (!globalLogging)
     return;

  VA_START(ap, format);
  if (vsnprintf(msg, MAX_BUF-1, format, ap) > (MAX_BUF - 2)) 
    msg[MAX_BUF-1] = '\0';
  va_end(ap);

  niso( 0, date );

#if 0
  while (pthread_mutex_trylock( &log_mutex ) != 0)
  {
     usleep( 100000 );
  }
#else
  pthread_mutex_lock( &log_mutex );
#endif

  if (UFDBglobalDebug || log == NULL) 
  {
    fprintf( stderr, "%s [%d] %s\n", date, globalPid, msg );
    fflush( stderr );
  }
  else
  {
    fd = log->fd;
    if (fd == NULL)
    {
      UFDBglobalDebug = 1;
      fprintf( stderr, "%s [%d] filedescriptor closed for %s\n", date, globalPid, log->name );
      fprintf( stderr, "%s [%d] %s\n", date, globalPid, msg );
      fflush( stderr );
    }
    else 
    {
      fprintf( fd, "%s [%d] %s\n", date, globalPid, msg );
#if defined(UFDB_FLUSH_ALL_LINES_IN_LOGFILE) || defined(DEBUG) || 1
      fflush( fd );
#endif

#ifdef UFDB_ROTATE_LOG_FILES
      if (--sizecheckcounter <= 0)
      {
	 sizecheckcounter = 100;		/* minimize the number of calls to ftell() */
	 pos = ftell( fd );
	 if (pos > UFDB_MAX_LOGFILE_SIZE)
	 {
	    strcpy( msg, log->name );
	    sgReopenLogfile( msg );
	 }
      }
#endif
    }
  }

  pthread_mutex_unlock( &log_mutex );
}


void ufdbLogError( char * format, ... )
{
  va_list ap;
  char    msg[MAX_BUF];

  if (!globalLogging)
     return;

  VA_START(ap, format);
  if (vsnprintf(msg, MAX_BUF-1, format, ap) > (MAX_BUF - 2)) 
     msg[MAX_BUF-1] = '\0';
  va_end(ap);

  ufdbLog( globalErrorLog, "%s", msg );
}


void ufdbLogFatalError( char * format, ... )
{
  va_list ap;
  char    msg[MAX_BUF];

  VA_START(ap, format);
  if (vsnprintf(msg, MAX_BUF-1, format, ap) > (MAX_BUF - 2)) 
     msg[MAX_BUF-1] = '\0';
  va_end(ap);

  ufdbLog( globalErrorLog, "%s", msg );

  globalFatalError = 1;
}



void ufdbLogRequest( struct LogFile * log,
		     struct SquidInfo * req,
		     struct Acl * acl, 
		     struct AclDest * aclpass,
		     struct sgRewrite * rewrite )
{
  char * ident = req->ident;
  char * srcDomain = req->srcDomain;
  char * srcclass;
  char * targetclass;
  char * rew;

  if (!globalLogging)
     return;

  if (rewrite == NULL) 
    rew = "-";
  else 
    rew = rewrite->name;

  if (*ident == '\0' || log->anonymous == 1)
    ident = "-";

  if (*srcDomain == '\0')
    srcDomain = "-";

  if (acl->source == NULL || acl->source->name == NULL)
    srcclass = "default";
  else
    srcclass = acl->source->name;

  if (aclpass == NULL)
    targetclass = "unknown";
  else if (aclpass->name == NULL) 
  {
    if (aclpass->type == ACL_TYPE_INADDR)
      targetclass = "in-addr";
    else if (aclpass->type == ACL_TYPE_TERMINATOR)
      targetclass = "none";
    else
      targetclass = "unknown";
  }
  else
    targetclass =  aclpass->name;

  ufdbLog( log->stat, "Request (%s/%s/%s) %s %s/%s %s %s",
	   srcclass, targetclass, rew, req->orig,
	   req->src, srcDomain, ident,
	   req->method
	  );
}

